#!/usr/bin/env python
# Do whatever you want with this code as long as you respect the GPLv2
# By: nikosapi (me@nikosapi.org)

import pygtk
import gtk, gtk.glade
import sys
import Image, ImageFilter
import StringIO
import time
from os.path import expanduser, isdir
from os import listdir

# The original number of times to blur the image
MAX_BLUR = 10

class enhance_gui(object):

    def __init__(self, image_file, original_blur = 10):
        self.gui = gtk.glade.XML('enhance.glade')
        self.gui.signal_connect('on_btn_enhance_clicked', self.on_btn_enhance_clicked)
        self.gui.signal_connect('on_main_window_destroy', gtk.main_quit)
        self.main_win = self.gui.get_widget('main_window')
        self.image_box = self.gui.get_widget('img_to_enhance')
        self.progressbar = self.gui.get_widget('progressbar')
        self.enhance_button = self.gui.get_widget('btn_enhance')
        self.enhance_button.set_tooltip_text('Use the enter button insead of the mouse for maximum 1337ness')

        self.original_blur = self.blur = original_blur
        self.original_image = Image.open(image_file)
        screen_dim = [ gtk.gdk.screen_width() - 50, gtk.gdk.screen_height() - 150]
        size = list(self.original_image.size)
        if size > screen_dim:
            new_size = [ min([screen_dim[0],screen_dim[1]*size[0]/size[1]]), min([screen_dim[1],screen_dim[0]*size[1]/size[0]]) ]
            print "Resizing image from %sx%s to %sx%s to fit %sx%s" % tuple(size + new_size + screen_dim)
            self.original_image = self.original_image.resize((new_size))
        self.image = None

        self.main_win.show()
        self.on_btn_enhance_clicked(None)
        self.progressbar.set_fraction(0)
        self.progressbar.set_text('Click ENHANCE to begin.')

    def on_btn_enhance_clicked(self, sender):
        self.enhance_button.set_sensitive(False)
        self.image = self.blur_image(self.original_image, self.original_blur, self.blur, self.progressbar)
        self.image_box.set_from_pixbuf(self.image_to_pixbuf(self.image))
        if self.blur >= 0:
            self.blur -= 1
            self.enhance_button.set_sensitive(True)
            self.enhance_button.grab_focus()
        else:
            self.progressbar.set_fraction(0)
            self.progressbar.set_text('Image ENHANCED.')

    def image_to_pixbuf(self, image):
        """ convert a PIL Image to pixbuf, from: 
            http://faq.pygtk.org/index.py?req=show&file=faq08.007.htp """
        fd = StringIO.StringIO()
        image.save(fd, "ppm")
        contents = fd.getvalue()
        fd.close()
        loader = gtk.gdk.PixbufLoader("pnm")
        loader.write(contents, len(contents))
        pixbuf = loader.get_pixbuf()
        loader.close()
        return pixbuf

    def blur_image(self, image, original_count, count, progress):
        """ Blur 'image' 'count' times """
        progress.set_text('Enhancing, please wait...')
        progress.set_fraction(0)
        wait_time = 0.1
        for i in range(original_count):
            while gtk.events_pending():
                gtk.main_iteration()
            if i <= count:
                orig_time = time.time()
                image = image.filter(ImageFilter.BLUR)
                wait_time = time.time() - orig_time
            elif i > count:
                time.sleep(wait_time)
            progress.set_fraction(float(i)/float(original_count))
        progress.set_fraction(1)
        progress.set_text('Enhancing complete!')
        return image


class file_chooser(object):

    def __init__(self):
        self.gui = gtk.glade.XML('enhance.glade')
        self.gui.signal_autoconnect(self)
        self.chooser_dialog = self.gui.get_widget('file_chooser')
        self.filename = ''

        # If the user has a ~/pictures directory start the file_chooser there
        dirs = [ 'pictures', 'photos', 'my pictures', 'my photos', 'pics' ]
        for f in listdir(expanduser('~')):
            if f.lower() in dirs and isdir(expanduser('~') + '/' + f):
                self.chooser_dialog.set_current_folder(expanduser('~') + '/' + f)
                break

    def run(self):
        exit = self.chooser_dialog.run()
        self.chooser_dialog.destroy()
        return self.filename * exit

    def on_btn_open_clicked(self, sender):
        self.filename = self.chooser_dialog.get_filename()
        self.chooser_dialog.response(True)

    def on_btn_cancel_clicked(self, sender):
        self.chooser_dialog.response(False)


if __name__ == "__main__":
    filename = file_chooser().run()
    if filename:
        main_window = enhance_gui(filename, MAX_BLUR)
        gtk.main()
    else:
        sys.exit(1)


